@@ -1,52 +1,8 @@
 #![allow(clippy::redundant_closure_call)]
 use tensor_rs::tensor::Tensor;
 use super::{OpTrait, OpHandle};
+use super::macros::new_binary_op;
 
-macro_rules! new_binary_op {
-    ($a:ident, $b:expr, $c:tt, $d: tt) => {
-        pub struct $a {
-            handle: OpHandle,
-        }
-        impl $a {
-            pub fn new() -> $a{
-                $a{
-                    handle: OpHandle::new(),
-                }
-            }
-        }
-        impl OpTrait for $a {
-            fn get_handle(&self) -> &OpHandle {
-                &self.handle
-            }
-            fn get_handle_mut(&mut self) -> &mut OpHandle {
-                &mut self.handle
-            }
-            fn get_name(&self) -> String {
-                ($b).to_string()
-            }
-            fn get_input_size(&self) -> usize {
-                2
-            }
-            fn get_output_size(&self) -> usize {
-                1
-            }
-            fn apply(&self, input: &[Tensor], output: &[Tensor]) {
-                $c(input, output)
-            }
-            fn grad(&self, input: &[Tensor], output_grad: &[Tensor], input_grad: &[Tensor]) {
-                $d(input, output_grad, input_grad)
-            }
-            fn get_values(&self) -> Vec<&Tensor> {
-                Vec::new()
-            }
-            fn get_grads(&self) -> Vec<&Tensor> {
-                Vec::new()
-            }
-            fn set_values(&self, _v: &[Tensor]) {
-            }
-        }
-    }
-}
 
 new_binary_op!(Add, "add",
                (|a:&[Tensor], b:&[Tensor]|
@@ -89,3 +45,30 @@ new_binary_op!(Div, "div",
                    input_grad[1].swap(&y);
                })
 );
+
+new_binary_op!(Matmul, "matmul",
+               (|a:&[Tensor], b:&[Tensor]|
+                b[0].swap(&a[0].matmul(&a[1]))),
+               (|input: &[Tensor], output_grad: &[Tensor], input_grad: &[Tensor]| {
+                   input_grad[0].swap(&input[1].outer(&output_grad[0], Some(true)));
+                   input_grad[1].swap(&input[0].outer(&output_grad[0], Some(true)));
+               })
+);
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::op::_gradient_checker;
+
+    #[test]
+    fn matmul() {
+        let mut op = Matmul::new();
+
+        //for i in 0..10 {
+        //    let zero = Tensor::from_vec_f64(&vec![(i - 5) as f64], &vec![1]);
+        //    let zero2 = zero.clone();
+        //    let good_grad = _gradient_checker(&mut op, &[zero, zero2], None, None, None);
+        //    assert_eq!(good_grad, true);                        
+        //}
+    }
+}
