@@ -67,6 +67,11 @@ impl OpHandle {
         }
     }
 }
+impl Default for OpHandle {
+    fn default() -> Self {
+        Self::new()
+    }
+}
 
 macro_rules! handle_method {
     () => {
@@ -299,7 +304,7 @@ impl OpCall for View {
 
         let op = Op::new(Rc::new(RefCell::new(Box::new(new_one))));
 
-        Ok(inputs[0].called_with(op, &inputs[1..inputs.len()])?)
+        inputs[0].called_with(op, &inputs[1..inputs.len()])
     }
 }
 impl OpTrait for View {
@@ -356,6 +361,8 @@ pub use nonlinear::{ELU, ReLU, Sigmoid};
 pub mod convolution;
 pub use convolution::{ Conv2d};
 
+pub mod pooling;
+
 pub mod loss;
 pub use loss::{MSELoss, BCEWithLogitsLoss, CrossEntropyLoss};
 
@@ -363,8 +370,18 @@ pub mod element;
 pub use element::{Abs, Acos, Asin, Atan, Ceil, Cos, Cosh, Exp, Expm1, Floor, Frac, Log, Log10, Log1p, Log1pexp, Log2, Neg, Reciprocal, Round, Rsqrt, //Sigmoid,
                   Sign, Sin, Sinh, Sqrt, Tan, Tanh, Trunc};
 
+pub mod comparison;
+pub use comparison::{MaxPair, MinPair, ArgSort, EqElem, Equal, Ge, Gt, Le, Lt, Ne};
+
 pub mod index_slicing;
 pub use index_slicing::{Cat, Chunk, ConditionalSelect, Gather, IndexSelect, IndexExclude, Reshape, Split, Squeeze, Stack, T, Take, Permute, Unsqueeze, Repeat};
 
 pub mod linalg;
-pub use linalg::{Det};
+pub use linalg::{Det, Inv, NormalizeUnit};
+
+pub mod reduction;
+pub use reduction::{Argmax, Argmin, Logsumexp, Mean, Prod, Std, Sum, Variance, Max, Min};
+
+pub mod vision;
+pub use vision::{GetPatch, };
+
