@@ -7,6 +7,7 @@ use tensor_rs::tensor::Tensor;
 use rand::prelude::StdRng;
 use crate::err::AutoDiffError;
 use super::compute_graph::Net;
+use crate::var::Var;
 
 /// Create random batch view from a large batch.
 pub struct MiniBatch {
@@ -21,7 +22,7 @@ impl MiniBatch {
         }
     }
 
-    pub fn next(&mut self, data: &Tensor, label: &Tensor) -> Result<(Tensor, Tensor), AutoDiffError> {
+    pub fn next(&mut self, data: &Var, label: &Var) -> Result<(Var, Var), AutoDiffError> {
         let sample_size = data.size()[0];
         let sample_size2 = label.size()[0];
 
@@ -29,10 +30,12 @@ impl MiniBatch {
             return Err(AutoDiffError::new(&format!("minibatch needs data and label has the same N {}, {}",
                                                    sample_size, sample_size2)));
         }
-        let index_t = Tensor::rand_usize(&mut self.rng, &[self.size], 0, sample_size);
+        let index_t = Var::rand_usize(&mut self.rng, &[self.size], 0, sample_size);
 
-        let mdata = data.index_select(0, &index_t);
-        let mlabel = label.index_select(0, &index_t);
+        let mdata = data.index_select(0, index_t.clone())?;
+        let mlabel = label.index_select(0, index_t)?;
+        mdata.reset_net();
+        mlabel.reset_net();
         Ok((mdata, mlabel))
     }
 }
@@ -46,7 +49,21 @@ pub struct SGD {
     lr: Tensor,
 }
 impl SGD {
+    #[cfg(feature = "use-f64")]
+    pub fn new(lr: f64) -> SGD {
+        Self::new_f64(lr)
+    }
+    #[cfg(feature = "use-f32")]
     pub fn new(lr: f32) -> SGD {
+        Self::new_f32(lr)
+    }
+    
+    pub fn new_f64(lr: f64) -> SGD {
+        SGD {
+            lr: Tensor::from_vec_f64(&[lr], &[1]),
+        }
+    }
+    pub fn new_f32(lr: f32) -> SGD {
         SGD {
             lr: Tensor::from_vec_f32(&[lr], &[1]),
         }
@@ -57,13 +74,12 @@ impl Optimizer for SGD {
         net.borrow_mut().visit_op(|x| {
             let weights = x.get_values();
             let grads = x.get_grads();
-            // println!("name: {}, {}, {}", x.get_name(), weights.len(), grads.len());
+            //println!("name: {}, {}, {}", x.get_name(), weights.len(), grads.len());
 
             let mut new_weight = Vec::new();
             for (i, j) in weights.iter().zip(grads.iter()) {
-                // println!("{:?}, {:?}, {:?}", i.size(), j.size(), self.lr.size());
-                
-                new_weight.push(i.add(&j.mul(&self.lr)));
+                //println!("{:?}, {:?}, {:?}", i.size(), j.size(), self.lr.size());
+                new_weight.push(i.sub(&j.mul(&self.lr)));
             }
             x.set_values(&new_weight);
         }, None, None);
