@@ -4,11 +4,11 @@ use std::fmt;
 use ::rand::prelude::StdRng;
 
 use tensor_rs::tensor::{Tensor};
-use crate::op::{Op
-};
+use crate::op::{Op};
 use crate::err::AutoDiffError;
 use crate::optim::Optimizer;
 use crate::var_inner::VarInner;
+use crate::compute_graph::{Net};
 
 
 macro_rules! var_1_to_1 {
@@ -42,6 +42,15 @@ macro_rules! var_more_to_1_with_para {
     }
 }
 
+macro_rules! var_1_to_1_with_para {
+    ($a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
+        pub fn $a(&self, $( $arg_name : $ArgTy ),*) -> Result<Var, AutoDiffError> {
+            Ok(Var {
+                var: Rc::new(RefCell::new(self.var.borrow().$a($( $arg_name ),*)?))})
+        }
+    }
+}
+
 macro_rules! delegate_new_op {
     ($a:ident, $( $arg_name:ident : $ArgTy:ty ),* $(,)?) => {
         pub fn $a($( $arg_name : $ArgTy ),*) -> Var {
@@ -52,27 +61,64 @@ macro_rules! delegate_new_op {
     }
 }
 
+/// [Var] can be thought as the value it holds plus a link
+/// to the computation graph.
+/// Majority of operators are methods on [Var].
 pub struct Var {
     var: Rc<RefCell<VarInner>>
 }
 impl Var {
     #[cfg(feature = "use-f64")]
     pub fn new(input: &[f64], dim: &[usize]) -> Var {
-        Var {
-            var: Rc::new(RefCell::new(VarInner::new(input, dim)))
-        }
+        Var::new_f64(input, dim)
     }
     #[cfg(feature = "use-f32")]
     pub fn new(input: &[f32], dim: &[usize]) -> Var {
+        Var::new_f32(input, dim)
+    }
+    pub fn new_f64(input: &[f64], dim: &[usize]) -> Var {
         Var {
-            var: Rc::new(RefCell::new(VarInner::new(input, dim)))
+            var: Rc::new(RefCell::new(VarInner::new_f64(input, dim)))
+        }
+    }
+    pub fn new_f32(input: &[f32], dim: &[usize]) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(VarInner::new_f32(input, dim)))
         }
     }
 
 
+    pub fn size(&self) -> Vec<usize> {
+        self.var.borrow().size()
+    }
+    pub fn numel(&self) -> usize {
+        self.var.borrow().numel()
+    }
+    pub fn get_f32(&self, o: &[usize]) -> f32 {
+        self.var.borrow().get_f32(o)
+    }
+    pub fn set_f32(&self, o: &[usize], v: f32) {
+        self.var.borrow_mut().set_f32(o, v);
+    }
+    pub fn get_f64(&self, o: &[usize]) -> f64 {
+        self.var.borrow().get_f64(o)
+    }
+    pub fn set_f64(&self, o: &[usize], v: f64) {
+        self.var.borrow_mut().set_f64(o, v);
+    }
 
+    pub fn set(&self, o: &Var) {
+        self.var.borrow_mut().set(&o.var.borrow());
+    }
 
-    //delegate_new_inner_op!(fill, dim: &[usize], fill_value: &);
+    pub fn fill(size: &[usize], fill_value: &Var) -> Var {
+        Var {
+            var: Rc::new(RefCell::new(
+                VarInner::fill(size, fill_value.var.clone())))
+        }
+    }
+    delegate_new_op!(fill_f32, size: &[usize], fill_value: f32);
+    delegate_new_op!(fill_f64, size: &[usize], fill_value: f64);
     delegate_new_op!(zeros, dim: &[usize]);
     delegate_new_op!(ones, dim: &[usize]);
     //delegate_new_inner_op!(arange, end: usize);
@@ -82,7 +128,14 @@ impl Var {
     delegate_new_op!(eye, n: usize, m: usize);
     delegate_new_op!(empty, dim: &[usize]);
     
-
+    /// Fill row by row.
+    pub fn from_record_f32(&self, row: usize, record: &[f32]) {
+        self.var.borrow().from_record_f32(row, record)
+    }
+    pub fn from_record_f64(&self, row: usize, record: &[f64]) {
+        self.var.borrow().from_record_f64(row, record)
+    }
+    
     // rand
     delegate_new_op!(rand_usize,
                      rng: &mut StdRng,
@@ -137,8 +190,22 @@ impl Var {
     var_2_to_1!(mul);
     var_2_to_1!(div);
     var_2_to_1!(matmul);
-    
+
+    // nonlinear
+    pub fn elu(&self, alpha: Var) -> Result<Var, AutoDiffError> {
+        Ok(Var {
+            var: Rc::new(RefCell::new(
+                self.var.borrow().elu(
+                    VarInner::new_tensor(alpha.val()))?))
+        })
+    }
+    var_1_to_1!(relu);
+    var_1_to_1!(sigmoid);
+
+    // loss
     var_2_to_1!(mse_loss);
+    var_2_to_1!(bce_with_logits_loss);
+    var_2_to_1!(cross_entropy_loss);
 
     //elementwise op
     var_1_to_1!(abs);
@@ -169,6 +236,19 @@ impl Var {
     var_1_to_1!(tanh);
     var_1_to_1!(trunc);
 
+    // comparison
+    var_2_to_1!(max_pair);
+    var_2_to_1!(min_pair);
+    var_1_to_1_with_para!(arg_sort, 
+                          dim: usize, descending: bool);
+    var_2_to_1!(eq_elem);
+    var_2_to_1!(equal);
+    var_2_to_1!(ge);
+    var_2_to_1!(gt);
+    var_2_to_1!(le);
+    var_2_to_1!(lt);
+    var_2_to_1!(ne);
+
     // index and slicing
     var_more_to_1_with_para!(cat, dim: usize);
     pub fn chunk(&self, chunks: usize, dim: usize)
@@ -192,7 +272,7 @@ impl Var {
     }
     pub fn gather(&self, dim: usize, index: Var)
                   -> Result<Var, AutoDiffError> {
-        let result = self.var.borrow().gather(dim, index.var.clone())?;
+        let result = self.var.borrow().gather(dim, index.var)?;
         Ok(Var {
             var: Rc::new(RefCell::new(result)),
         })
@@ -201,7 +281,7 @@ impl Var {
                         index: Var)
                         -> Result<Var, AutoDiffError> {
         let result = self.var.borrow().index_select(
-            dim, index.var.clone())?;
+            dim, index.var)?;
         Ok(Var {
             var: Rc::new(RefCell::new(result)),
         })
@@ -210,7 +290,7 @@ impl Var {
                         index: Var)
                         -> Result<Var, AutoDiffError> {
         let result = self.var.borrow().index_exclude(
-            dim, index.var.clone())?;
+            dim, index.var)?;
         Ok(Var {
             var: Rc::new(RefCell::new(result)),
         })
@@ -273,6 +353,24 @@ impl Var {
 
     // linalg
     var_1_to_1!(det);
+    var_1_to_1!(inv);
+    var_1_to_1!(normalize_unit);
+
+    // reduction
+    var_1_to_1_with_para!(argmax, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(argmin, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(logsumexp, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(mean, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(prod, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(std, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(sum, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(var, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(max, dim: Option<&[usize]>, keepdim: bool);
+    var_1_to_1_with_para!(min, dim: Option<&[usize]>, keepdim: bool);
+
+    // images
+    var_1_to_1_with_para!(get_patch, range: &[(usize, usize)], step: Option<&[usize]>);
+    var_1_to_1_with_para!(view, new_shape: &[usize]);
 
 
     // innternal use
@@ -280,11 +378,17 @@ impl Var {
         self.var.borrow().val()
     }
 
-        /// Use gradient or not, default is to use.
+    /// Use gradient or not, default is to use.
     pub fn set_grad(&self, use_gradient: bool) {
         self.var.borrow_mut().set_grad(use_gradient);
     }
 
+    /// Reset net in the background.
+    pub fn reset_net(&self) {
+        self.var.borrow_mut().reset_net();
+    }
+
+    /// The current gradient for the Var.
     pub fn grad(&self) -> Result<Var, AutoDiffError> {
         Ok(Var {
             var: Rc::new(RefCell::new(self.var.borrow().grad()?))
@@ -298,22 +402,26 @@ impl Var {
     }
 
     pub fn step(&self, opt: &mut dyn Optimizer) -> Result<(), AutoDiffError> {
-        Ok(self.var.borrow().step(opt)?)
+        self.var.borrow().step(opt)
     }
 
     pub fn rerun(&self) -> Result<(), AutoDiffError> {
-        Ok(self.var.borrow().rerun()?)
+        self.var.borrow().rerun()
     }
 
     pub(crate) fn called_with(&self, op: Op,
                               others: &[&Var]) -> Result<Vec<Var>, AutoDiffError> {
-        let mut refs: Vec<Rc<RefCell<VarInner>>> = others.iter().map(|x| x.var.clone()).collect();
-        let mut var_inners = self.var.borrow().called_with(op, &mut refs)?;
+        let refs: Vec<Rc<RefCell<VarInner>>> = others.iter().map(|x| x.var.clone()).collect();
+        let mut var_inners = self.var.borrow().called_with(op, &refs)?;
         let ret: Vec<Var> = var_inners.drain(..).map(|x| Var {
             var: Rc::new(RefCell::new(x))
         }).collect();
         Ok(ret)
     }
+
+    pub fn dump_net(&self) -> Rc<RefCell<Net>> {
+        self.var.borrow().dump_net()
+    }
 }
 
 impl PartialEq for Var {
